"use strict";

import { createHighlighterCore, normalizeTheme } from '@shikijs/core';
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript';
import React, { useMemo } from 'react';
import { ActivityIndicator, ScrollView, StyleSheet, Text, View } from 'react-native';
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
let highlighterInstance = null;
async function getHighlighter() {
  if (!highlighterInstance) {
    highlighterInstance = await createHighlighterCore({
      engine: createJavaScriptRegexEngine()
    });
  }
  return highlighterInstance;
}
export const SyntaxHighlighter = ({
  text,
  language,
  theme,
  fontSize = 14,
  textStyle,
  scrollViewProps,
  containerStyle
}) => {
  const [tokens, setTokens] = React.useState([]);
  const [loading, setLoading] = React.useState(true);
  const [error, setError] = React.useState(null);

  // Normalize theme
  const normalizedTheme = useMemo(() => {
    console.log('Raw theme:', theme);
    const normalized = normalizeTheme(theme);
    console.log('Normalized theme:', normalized);
    return normalized;
  }, [theme]);

  // Inside highlight function:
  React.useEffect(() => {
    const highlight = async () => {
      try {
        setLoading(true);
        const highlighter = await getHighlighter();

        // Log theme info before loading
        console.log('Theme before loading:', {
          name: normalizedTheme.name,
          settings: normalizedTheme.settings?.length,
          tokenColors: normalizedTheme.tokenColors?.length
        });

        // Load language and theme
        if (typeof language !== 'string') {
          await highlighter.loadLanguage(language);
        }
        await highlighter.loadTheme(normalizedTheme);

        // Log loaded languages and themes
        console.log('Loaded languages:', highlighter.getLoadedLanguages());
        console.log('Loaded themes:', highlighter.getLoadedThemes());

        // Get theme by name to verify it's loaded
        const loadedTheme = highlighter.getTheme(normalizedTheme.name);
        console.log('Loaded theme details:', {
          name: loadedTheme.name,
          settings: loadedTheme.settings?.length
        });

        // Highlight with debug info
        const result = highlighter.codeToTokens(text, {
          lang: typeof language === 'string' ? language : language.name,
          theme: normalizedTheme.name,
          includeExplanation: true
        });

        // Log detailed token info
        console.log('First line tokens:', result.tokens[0]?.map(token => ({
          content: token.content,
          color: token.color,
          fontStyle: token.fontStyle,
          scopes: token.explanation?.[0]?.scopes.map(s => ({
            scopeName: s.scopeName,
            themeMatches: s.themeMatches?.length
          }))
        })));
        setTokens(result.tokens);
        setError(null);
      } catch (err) {
        console.error('Highlighting error:', err);
        setError(err instanceof Error ? err.message : 'Failed to highlight code');
      } finally {
        setLoading(false);
      }
    };
    highlight();
  }, [text, language, normalizedTheme]);
  const renderToken = (token, index) => {
    const styles = [{
      fontSize
    }];
    if (textStyle) {
      styles.push(textStyle);
    }

    // Add color from token
    if (token.color) {
      styles.push({
        color: token.color
      });
    }

    // Add font styles
    if (token.fontStyle === 1) styles.push({
      fontStyle: 'italic'
    });
    if (token.fontStyle === 2) styles.push({
      fontWeight: 'bold'
    });
    if (token.fontStyle === 4) styles.push({
      textDecorationLine: 'underline'
    });

    // If token has no color, use theme's foreground color
    if (!token.color && normalizedTheme.fg) {
      styles.push({
        color: normalizedTheme.fg
      });
    }
    const finalStyle = StyleSheet.flatten(styles);
    console.log('Token style:', {
      content: token.content,
      color: token.color || normalizedTheme.fg,
      styles: finalStyle
    });
    return /*#__PURE__*/_jsx(Text, {
      style: finalStyle,
      children: token.content
    }, `${token.offset}-${index}`);
  };
  const renderLine = (lineTokens, index) => /*#__PURE__*/_jsx(View, {
    style: {
      flexDirection: 'row',
      flexWrap: 'wrap'
    },
    children: lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))
  }, index);
  if (loading) {
    return /*#__PURE__*/_jsx(View, {
      style: [{
        backgroundColor: normalizedTheme.bg,
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/_jsx(ActivityIndicator, {})
    });
  }
  if (error) {
    return /*#__PURE__*/_jsx(View, {
      style: [{
        backgroundColor: normalizedTheme.bg,
        padding: 10
      }, containerStyle],
      children: /*#__PURE__*/_jsxs(Text, {
        style: {
          color: '#ff6b6b'
        },
        children: ["Error:", error]
      })
    });
  }
  return /*#__PURE__*/_jsx(ScrollView, {
    ...scrollViewProps,
    horizontal: true,
    style: [{
      backgroundColor: normalizedTheme.bg
    }, scrollViewProps?.style],
    contentContainerStyle: [containerStyle],
    children: /*#__PURE__*/_jsx(View, {
      onStartShouldSetResponder: () => true,
      children: tokens.map((lineTokens, index) => renderLine(lineTokens, index))
    })
  });
};
export default SyntaxHighlighter;
//# sourceMappingURL=SyntaxHighlighter.js.map