import type { LanguageRegistration, ThemedToken, ThemeRegistrationAny } from '@shikijs/types'
import { createHighlighterCore, normalizeTheme } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { type FunctionComponent, useMemo } from 'react'
import { ActivityIndicator, ScrollView, type ScrollViewProps, type StyleProp, StyleSheet, Text, type TextStyle, View, type ViewStyle } from 'react-native'

export interface SyntaxHighlighterProps {
  text: string
  language: string | LanguageRegistration
  theme: ThemeRegistrationAny
  fontSize?: number
  textStyle?: StyleProp<TextStyle>
  scrollViewProps?: ScrollViewProps
  containerStyle?: StyleProp<ViewStyle>
}

let highlighterInstance: Awaited<ReturnType<typeof createHighlighterCore>> | null = null

async function getHighlighter() {
  if (!highlighterInstance) {
    highlighterInstance = await createHighlighterCore({
      engine: createJavaScriptRegexEngine(),
    })
  }
  return highlighterInstance
}

export const SyntaxHighlighter: FunctionComponent<SyntaxHighlighterProps> = ({ text, language, theme, fontSize = 14, textStyle, scrollViewProps, containerStyle }) => {
  const [tokens, setTokens] = React.useState<ThemedToken[][]>([])
  const [loading, setLoading] = React.useState(true)
  const [error, setError] = React.useState<string | null>(null)

  // Normalize theme
  const normalizedTheme = useMemo(() => {
    console.log('Raw theme:', theme)
    const normalized = normalizeTheme(theme)
    console.log('Normalized theme:', normalized)
    return normalized
  }, [theme])

  // Inside highlight function:
  React.useEffect(() => {
    const highlight = async () => {
      try {
        setLoading(true)
        const highlighter = await getHighlighter()

        // Log theme info before loading
        console.log('Theme before loading:', {
          name: normalizedTheme.name,
          settings: normalizedTheme.settings?.length,
          tokenColors: normalizedTheme.tokenColors?.length,
        })

        // Load language and theme
        if (typeof language !== 'string') {
          await highlighter.loadLanguage(language)
        }
        await highlighter.loadTheme(normalizedTheme)

        // Log loaded languages and themes
        console.log('Loaded languages:', highlighter.getLoadedLanguages())
        console.log('Loaded themes:', highlighter.getLoadedThemes())

        // Get theme by name to verify it's loaded
        const loadedTheme = highlighter.getTheme(normalizedTheme.name)
        console.log('Loaded theme details:', {
          name: loadedTheme.name,
          settings: loadedTheme.settings?.length,
        })

        // Highlight with debug info
        const result = highlighter.codeToTokens(text, {
          lang: typeof language === 'string' ? language : language.name,
          theme: normalizedTheme.name,
          includeExplanation: true,
        })

        // Log detailed token info
        console.log(
          'First line tokens:',
          result.tokens[0]?.map(token => ({
            content: token.content,
            color: token.color,
            fontStyle: token.fontStyle,
            scopes: token.explanation?.[0]?.scopes.map(s => ({
              scopeName: s.scopeName,
              themeMatches: s.themeMatches?.length,
            })),
          })),
        )

        setTokens(result.tokens)
        setError(null)
      }
      catch (err) {
        console.error('Highlighting error:', err)
        setError(err instanceof Error ? err.message : 'Failed to highlight code')
      }
      finally {
        setLoading(false)
      }
    }

    highlight()
  }, [text, language, normalizedTheme])

  const renderToken = (token: ThemedToken, index: number) => {
    const styles: TextStyle[] = [{ fontSize }]
    if (textStyle) {
      styles.push(textStyle as TextStyle)
    }

    // Add color from token
    if (token.color) {
      styles.push({ color: token.color })
    }

    // Add font styles
    if (token.fontStyle === 1)
      styles.push({ fontStyle: 'italic' })
    if (token.fontStyle === 2)
      styles.push({ fontWeight: 'bold' })
    if (token.fontStyle === 4)
      styles.push({ textDecorationLine: 'underline' })

    // If token has no color, use theme's foreground color
    if (!token.color && normalizedTheme.fg) {
      styles.push({ color: normalizedTheme.fg })
    }

    const finalStyle = StyleSheet.flatten(styles)

    console.log('Token style:', {
      content: token.content,
      color: token.color || normalizedTheme.fg,
      styles: finalStyle,
    })

    return (
      <Text key={`${token.offset}-${index}`} style={finalStyle}>
        {token.content}
      </Text>
    )
  }

  const renderLine = (lineTokens: ThemedToken[], index: number) => (
    <View key={index} style={{ flexDirection: 'row', flexWrap: 'wrap' }}>
      {lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))}
    </View>
  )

  if (loading) {
    return (
      <View style={[{ backgroundColor: normalizedTheme.bg, padding: 10 }, containerStyle]}>
        <ActivityIndicator />
      </View>
    )
  }

  if (error) {
    return (
      <View style={[{ backgroundColor: normalizedTheme.bg, padding: 10 }, containerStyle]}>
        <Text style={{ color: '#ff6b6b' }}>
          Error:
          {error}
        </Text>
      </View>
    )
  }

  return (
    <ScrollView {...scrollViewProps} horizontal style={[{ backgroundColor: normalizedTheme.bg }, scrollViewProps?.style]} contentContainerStyle={[containerStyle]}>
      <View onStartShouldSetResponder={() => true}>{tokens.map((lineTokens, index) => renderLine(lineTokens, index))}</View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
